@@ -19,7 +19,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: xvid4.h,v 1.3 2003/12/05 16:23:29 tibit Exp $
+ * $Id: xvid4.h,v 1.4 2004/01/03 16:51:45 tibit Exp $
  *
  ****************************************************************************/
 
@@ -57,7 +57,7 @@ extern "C" {
 #define XVID_API_MAJOR(a)        (((a)>>16) & 0xff)
 #define XVID_API_MINOR(a)        (((a)>> 0) & 0xff)
 
-#define XVID_VERSION             XVID_MAKE_VERSION(1,0,-127)
+#define XVID_VERSION             XVID_MAKE_VERSION(1,0,-126)
 #define XVID_API                 XVID_MAKE_API(4, 0)
 
 #define XVID_UNSTABLE
@@ -73,7 +73,7 @@ extern "C" {
  * doesnt hurt but not increasing it could cause difficulty for decoders in the
  * future
  */
-#define XVID_BS_VERSION "0023"
+#define XVID_BS_VERSION "0025"
 
 
 /*****************************************************************************
@@ -96,15 +96,16 @@ extern "C" {
 
 /* colorspace values */
 
-#define XVID_CSP_USER     (1<< 0) /* 4:2:0 planar */
-#define XVID_CSP_I420     (1<< 1) /* 4:2:0 packed(planar win32) */
-#define XVID_CSP_YV12     (1<< 2) /* 4:2:0 packed(planar win32) */
+#define XVID_CSP_USER     (1<< 0) /* 4:2:0 planar (==I420, except for pointers/strides) */
+#define XVID_CSP_I420     (1<< 1) /* 4:2:0 planar */
+#define XVID_CSP_YV12     (1<< 2) /* 4:2:0 planar */
 #define XVID_CSP_YUY2     (1<< 3) /* 4:2:2 packed */
 #define XVID_CSP_UYVY     (1<< 4) /* 4:2:2 packed */
 #define XVID_CSP_YVYU     (1<< 5) /* 4:2:2 packed */
 #define XVID_CSP_BGRA     (1<< 6) /* 32-bit bgra packed */
 #define XVID_CSP_ABGR     (1<< 7) /* 32-bit abgr packed */
 #define XVID_CSP_RGBA     (1<< 8) /* 32-bit rgba packed */
+#define XVID_CSP_ARGB     (1<<15) /* 32-bit argb packed */
 #define XVID_CSP_BGR      (1<< 9) /* 24-bit bgr packed */
 #define XVID_CSP_RGB555   (1<<10) /* 16-bit rgb555 packed */
 #define XVID_CSP_RGB565   (1<<11) /* 16-bit rgb565 packed */
@@ -138,7 +139,7 @@ typedef struct {
 #define XVID_PROFILE_AS_L4   0xf4
 
 /* aspect ratios */
-#define XVID_PAR_11_VGA    1 /* 1:1 vga (square), default if AR is not precised (ie: ==0) */
+#define XVID_PAR_11_VGA    1 /* 1:1 vga (square), default if supplied PAR is not a valid value */
 #define XVID_PAR_43_PAL    2 /* 4:3 pal (12:11 625-line) */
 #define XVID_PAR_43_NTSC   3 /* 4:3 ntsc (10:11 525-line) */
 #define XVID_PAR_169_PAL   4 /* 16:9 pal (16:11 625-line) */
@@ -218,7 +219,7 @@ typedef struct {
 #define XVID_GBL_INFO    1 /* return some info about xvidcore, and the host computer */
 #define XVID_GBL_CONVERT 2 /* colorspace conversion utility */
 
-int xvid_global(void *handle, int opt, void *param1, void *param2);
+extern int xvid_global(void *handle, int opt, void *param1, void *param2);
 
 
 /*****************************************************************************
@@ -229,7 +230,7 @@ int xvid_global(void *handle, int opt, void *param1, void *param2);
 #define XVID_DEC_DESTROY 1 /* destroy decore instance: return 0 on success */
 #define XVID_DEC_DECODE  2 /* decode a frame: returns number of bytes consumed >= 0 */
 
-int xvid_decore(void *handle, int opt, void *param1, void *param2);
+extern int xvid_decore(void *handle, int opt, void *param1, void *param2);
 
 /* XVID_DEC_CREATE param 1
 	image width & height may be specified here when the dimensions are
@@ -246,6 +247,9 @@ typedef struct {
 /* general flags */
 #define XVID_LOWDELAY      (1<<0) /* lowdelay mode  */
 #define XVID_DISCONTINUITY (1<<1) /* indicates break in stream */
+#define XVID_DEBLOCKY      (1<<2) /* perform luma deblocking */
+#define XVID_DEBLOCKUV     (1<<3) /* perform chroma deblocking */
+#define XVID_FILMEFFECT    (1<<4) /* adds film grain */
 
 typedef struct {
 	int version;
@@ -277,15 +281,13 @@ typedef struct
 			int general;        /* [out] flags */
 			int width;          /* [out] width */
 			int height;         /* [out] height */
-			int par;            /* [out] picture aspect ratio (refer to XVID_PAR_xxx above) */
-			int par_width;      /* [out] aspect ratio width */
-			int par_height;     /* [out] aspect ratio height */
+			int par;            /* [out] pixel aspect ratio (refer to XVID_PAR_xxx above) */
+			int par_width;      /* [out] aspect ratio width  [1..255] */
+			int par_height;     /* [out] aspect ratio height [1..255] */
 		} vol;
 	} data;
 } xvid_dec_stats_t;
 
-
-
 #define XVID_ZONE_QUANT  (1<<0)
 #define XVID_ZONE_WEIGHT (1<<1)
 
@@ -298,6 +300,37 @@ typedef struct
 } xvid_enc_zone_t;
 
 
+/*----------------------------------------------------------------------------
+ * xvid_enc_stats_t structure
+ *
+ * Used in:
+ *  - xvid_plg_data_t structure
+ *  - optional parameter in xvid_encore() function
+ *
+ * .coding_type = XVID_TYPE_NOTHING if the stats are not given
+ *--------------------------------------------------------------------------*/
+
+typedef struct {
+	int version;
+
+	/* encoding parameters */
+	int type;      /* [out] coding type */
+	int quant;     /* [out] frame quantizer */
+	int vol_flags; /* [out] vol flags (see above) */
+	int vop_flags; /* [out] vop flags (see above) */
+
+	/* bitrate */
+	int length;    /* [out] frame length */
+
+	int hlength;   /* [out] header length (bytes) */
+	int kblks;     /* [out] number of blocks compressed as Intra */
+	int mblks;     /* [out] number of blocks compressed as Inter */
+	int ublks;     /* [out] number of blocks marked as not_coded */
+
+	int sse_y;     /* [out] Y plane's sse */
+	int sse_u;     /* [out] U plane's sse */
+	int sse_v;     /* [out] V plane's sse */
+} xvid_enc_stats_t;
 
 /*****************************************************************************
   xvid plugin system -- internals
@@ -354,7 +387,6 @@ typedef struct
 	int num_frames; /* [out] total frame encoded */
 } xvid_plg_destroy_t;
 
-
 typedef struct
 {
 	int version;
@@ -386,6 +418,8 @@ typedef struct
 	int vol_flags;          /* [in,out] */
 	int motion_flags;       /* [in,out] */
 
+/* Deprecated, use the stats field instead.
+ * Will disapear before 1.0 */
 	int length;             /* [out] after: length of encoded frame */
 	int kblks;              /* [out] number of blocks compressed as Intra */
 	int mblks;              /* [out] number of blocks compressed as Inter */
@@ -393,10 +427,13 @@ typedef struct
 	int sse_y;              /* [out] Y plane's sse */
 	int sse_u;              /* [out] U plane's sse */
 	int sse_v;              /* [out] V plane's sse */
+/* End of duplicated data, kept only for binary compatibility */
+
 	int bquant_ratio;       /* [in] */
 	int bquant_offset;      /* [in] */
-} xvid_plg_data_t;
 
+	xvid_enc_stats_t stats; /* [out] frame statistics */
+} xvid_plg_data_t;
 
 /*****************************************************************************
   xvid plugin system -- external
@@ -427,14 +464,14 @@ typedef struct
 } xvid_enc_plugin_t;
 
 
-xvid_plugin_func xvid_plugin_single;   /* single-pass rate control */
-xvid_plugin_func xvid_plugin_2pass1;   /* two-pass rate control: first pass */
-xvid_plugin_func xvid_plugin_2pass2;   /* two-pass rate control: second pass */
+extern xvid_plugin_func xvid_plugin_single;   /* single-pass rate control */
+extern xvid_plugin_func xvid_plugin_2pass1;   /* two-pass rate control: first pass */
+extern xvid_plugin_func xvid_plugin_2pass2;   /* two-pass rate control: second pass */
 
-xvid_plugin_func xvid_plugin_lumimasking;  /* lumimasking */
+extern xvid_plugin_func xvid_plugin_lumimasking;  /* lumimasking */
 
-xvid_plugin_func xvid_plugin_psnr;	/* write psnr values to stdout */
-xvid_plugin_func xvid_plugin_dump;	/* dump before and after yuvpgms */
+extern xvid_plugin_func xvid_plugin_psnr;	/* write psnr values to stdout */
+extern xvid_plugin_func xvid_plugin_dump;	/* dump before and after yuvpgms */
 
 
 /* single pass rate control
@@ -482,22 +519,58 @@ typedef struct {
 	int container_frame_overhead; /* [in] How many bytes the controller has to compensate per frame due to container format overhead */
 }xvid_plugin_2pass2_t;
 
-
 /*****************************************************************************
- * xvid_encore()
+ *                             ENCODER API
  ****************************************************************************/
 
-/* Encoder options */
+/*----------------------------------------------------------------------------
+ * Encoder operations
+ *--------------------------------------------------------------------------*/
+
 #define XVID_ENC_CREATE  0 /* create encoder instance; returns 0 on success */
 #define XVID_ENC_DESTROY 1 /* destroy encoder instance; returns 0 on success */
 #define XVID_ENC_ENCODE  2 /* encode a frame: returns number of ouput bytes
                             * 0 means this frame should not be written (ie. encoder lag) */
 
-int xvid_encore(void *handle, int opt, void *param1, void *param2);
+
+/*----------------------------------------------------------------------------
+ * Encoder entry point
+ *--------------------------------------------------------------------------*/
+
+extern int xvid_encore(void *handle, int opt, void *param1, void *param2);
+
+/* Quick API reference
+ *
+ * XVID_ENC_CREATE operation
+ *  - handle: ignored
+ *  - opt: XVID_ENC_CREATE
+ *  - param1: address of a xvid_enc_create_t structure
+ *  - param2: ignored
+ *
+ * XVID_ENC_ENCODE operation
+ *  - handle: an instance returned by a CREATE op
+ *  - opt: XVID_ENC_ENCODE
+ *  - param1: address of a xvid_enc_frame_t structure
+ *  - param2: address of a xvid_enc_stats_t structure (optional)
+ *            its return value is asynchronous to what is written to the buffer
+ *            depending on the delay introduced by bvop use. It's display
+ *            ordered.
+ *
+ * XVID_ENC_DESTROY operation
+ *  - handle: an instance returned by a CREATE op
+ *  - opt: XVID_ENC_DESTROY
+ *  - param1: ignored
+ *  - param2: ignored
+ */
 
 
+/*----------------------------------------------------------------------------
+ * "Global" flags
+ *
+ * These flags are used for xvid_enc_create_t->global field during instance
+ * creation (operation XVID_ENC_CREATE)
+ *--------------------------------------------------------------------------*/
 
-/* Global Flags */
 #define XVID_GLOBAL_PACKED            (1<<0) /* packed bitstream */
 #define XVID_GLOBAL_CLOSED_GOP        (1<<1) /* closed_gop:	was DX50BVOP dx50 bvop compatibility */
 #define XVID_GLOBAL_EXTRASTATS_ENABLE (1<<2)
@@ -507,8 +580,13 @@ int xvid_encore(void *handle, int opt, void *param1, void *param2);
 #endif
 
 
-/* XVID_ENC_ENCODE param1 */
-/* vol-based flags */
+/*----------------------------------------------------------------------------
+ * "VOL" flags
+ *
+ * These flags are used for xvid_enc_frame_t->vol_flags field during frame
+ * encoding (operation XVID_ENC_ENCODE)
+ *--------------------------------------------------------------------------*/
+
 #define XVID_VOL_MPEGQUANT      (1<<0) /* enable MPEG type quantization */
 #define XVID_VOL_EXTRASTATS     (1<<1) /* enable plane sse stats */
 #define XVID_VOL_QUARTERPEL     (1<<2) /* enable quarterpel: frames will encoded as quarterpel */
@@ -517,32 +595,47 @@ int xvid_encore(void *handle, int opt, void *param1, void *param2);
 #define XVID_VOL_INTERLACING    (1<<5) /* enable interlaced encoding */
 
 
+/*----------------------------------------------------------------------------
+ * "VOP" flags
+ *
+ * These flags are used for xvid_enc_frame_t->vop_flags field during frame
+ * encoding (operation XVID_ENC_ENCODE)
+ *--------------------------------------------------------------------------*/
+
+/* Always valid */
+#define XVID_VOP_DEBUG                (1<< 0) /* print debug messages in frames */
+#define XVID_VOP_HALFPEL              (1<< 1) /* use halfpel interpolation */
+#define XVID_VOP_INTER4V              (1<< 2) /* use 4 motion vectors per MB */
+#define XVID_VOP_TRELLISQUANT         (1<< 3) /* use trellis based R-D "optimal" quantization */
+#define XVID_VOP_CHROMAOPT            (1<< 4) /* enable chroma optimization pre-filter */
+#define XVID_VOP_CARTOON              (1<< 5) /* use 'cartoon mode' */
+#define XVID_VOP_GREYSCALE            (1<< 6) /* enable greyscale only mode (even for  color input material chroma is ignored) */
+#define XVID_VOP_HQACPRED             (1<< 7) /* high quality ac prediction */
+#define XVID_VOP_MODEDECISION_RD      (1<< 8) /* enable DCT-ME and use it for mode decision */
+#define XVID_VOP_FAST_MODEDECISION_RD (1<<12) /* use simplified R-D mode decision */
+
+/* Only valid for vol_flags|=XVID_VOL_INTERLACING */
+#define XVID_VOP_TOPFIELDFIRST        (1<< 9) /* set top-field-first flag  */
+#define XVID_VOP_ALTERNATESCAN        (1<<10) /* set alternate vertical scan flag */
 
-/* vop based flags */
-#define XVID_VOP_DEBUG           (1<< 0)
-#define XVID_VOP_HALFPEL         (1<< 1) /* use halfpel interpolation */
-#define XVID_VOP_INTER4V         (1<< 2)
-#define XVID_VOP_TRELLISQUANT    (1<< 3) /* use trellis based R-D "optimal" quantization */
-#define XVID_VOP_CHROMAOPT       (1<< 4) /* enable chroma optimization pre-filter */
-#define XVID_VOP_CARTOON         (1<< 5) /* use 'cartoon mode' */
-#define XVID_VOP_GREYSCALE       (1<< 6) /* enable greyscale only mode (even for  color input material chroma is ignored) */
-#define XVID_VOP_HQACPRED        (1<< 7) /* high quality ac prediction */
-#define XVID_VOP_MODEDECISION_RD (1<< 8) /* enable DCT-ME and use it for mode decision */
-/* only valid for vol_flags|=XVID_VOL_INTERLACING */
-#define XVID_VOP_TOPFIELDFIRST   (1<< 9) /* set top-field-first flag  */
-#define XVID_VOP_ALTERNATESCAN   (1<<10) /* set alternate vertical scan flag */
 /* only valid for vol_flags|=XVID_VOL_REDUCED_ENABLED */
-#define XVID_VOP_REDUCED         (1<<11) /* reduced resolution vop */
-
-#define XVID_VOP_FAST_MODEDECISION_RD (1<< 12) /* use simplified R-D mode decision */
+#define XVID_VOP_REDUCED              (1<<11) /* reduced resolution vop */
 
 
+/*----------------------------------------------------------------------------
+ * "Motion" flags
+ *
+ * These flags are used for xvid_enc_frame_t->motion field during frame
+ * encoding (operation XVID_ENC_ENCODE)
+ *--------------------------------------------------------------------------*/
 
-/* Motion based flags */
+/* Motion Estimation Search Patterns */
 #define XVID_ME_ADVANCEDDIAMOND16     (1<< 0) /* use advdiamonds instead of diamonds as search pattern */
 #define XVID_ME_ADVANCEDDIAMOND8      (1<< 1) /* use advdiamond for XVID_ME_EXTSEARCH8 */
 #define XVID_ME_USESQUARES16          (1<< 2) /* use squares instead of diamonds as search pattern */
 #define XVID_ME_USESQUARES8           (1<< 3) /* use square for XVID_ME_EXTSEARCH8 */
+
+/* SAD operator based flags */
 #define XVID_ME_HALFPELREFINE16       (1<< 4)
 #define XVID_ME_HALFPELREFINE8        (1<< 6)
 #define XVID_ME_QUARTERPELREFINE16    (1<< 7)
@@ -552,8 +645,11 @@ int xvid_encore(void *handle, int opt, void *param1, void *param2);
 #define XVID_ME_EXTSEARCH8            (1<<11) /* use diamond/square for extended 8x8 search */
 #define XVID_ME_CHROMA_PVOP           (1<<12) /* also use chroma for P_VOP/S_VOP ME */
 #define XVID_ME_CHROMA_BVOP           (1<<13) /* also use chroma for B_VOP ME */
+#define XVID_ME_FASTREFINE16          (1<<25) /* use low-complexity refinement functions */
+#define XVID_ME_FASTREFINE8           (1<<29) /* low-complexity 8x8 sub-block refinement */
 
-/* Motion search using DCT. use XVID_VOP_MODEDECISION_BITS to enable */
+/* Rate Distortion based flags
+ * Valid when XVID_VOP_MODEDECISION_RD is enabled */
 #define XVID_ME_HALFPELREFINE16_RD    (1<<14) /* perform RD-based halfpel refinement */
 #define XVID_ME_HALFPELREFINE8_RD     (1<<15) /* perform RD-based halfpel refinement for 8x8 mode */
 #define XVID_ME_QUARTERPELREFINE16_RD (1<<16) /* perform RD-based qpel refinement */
@@ -561,15 +657,26 @@ int xvid_encore(void *handle, int opt, void *param1, void *param2);
 #define XVID_ME_EXTSEARCH_RD          (1<<18) /* perform RD-based search using square pattern enable XVID_ME_EXTSEARCH8 to do this in 8x8 search as well */
 #define XVID_ME_CHECKPREDICTION_RD    (1<<19) /* always check vector equal to prediction */
 
+/* Other */
+#define XVID_ME_DETECT_STATIC_MOTION  (1<<24) /* speed-up ME by detecting stationary scenes */
+#define XVID_ME_SKIP_DELTASEARCH      (1<<26) /* speed-up by skipping b-frame delta search */
+#define XVID_ME_FAST_MODEINTERPOLATE  (1<<27) /* speed-up by partly skipping interpolate mode */
+#define XVID_ME_BFRAME_EARLYSTOP      (1<<28) /* speed-up by early exiting b-search */
+
+/* Unused */
 #define XVID_ME_UNRESTRICTED16        (1<<20) /* unrestricted ME, not implemented */
 #define XVID_ME_OVERLAPPING16         (1<<21) /* overlapping ME, not implemented */
 #define XVID_ME_UNRESTRICTED8         (1<<22) /* unrestricted ME, not implemented */
 #define XVID_ME_OVERLAPPING8          (1<<23) /* overlapping ME, not implemented */
 
-#define XVID_ME_DETECT_STATIC_MOTION  (1<<24) /* speed-up ME by detecting stationary scenes */
-#define XVID_ME_FASTREFINE16		  (1<<25) /* use low-complexity refinement functions */
 
-/* XVID_ENC_CREATE param1 */
+/*----------------------------------------------------------------------------
+ * xvid_enc_create_t structure definition
+ *
+ * This structure is passed as param1 during an instance creation (operation
+ * XVID_ENC_CREATE)
+ *--------------------------------------------------------------------------*/
+
 typedef struct {
 	int version;
 
@@ -606,24 +713,39 @@ typedef struct {
 	/* ---------------------------------------------- */
 
 	void *handle;                /* [out] encoder instance handle */
-}
-xvid_enc_create_t;
+} xvid_enc_create_t;
 
 
+/*----------------------------------------------------------------------------
+ * xvid_enc_frame_t structure definition
+ *
+ * This structure is passed as param1 during a frame encoding (operation
+ * XVID_ENC_ENCODE)
+ *--------------------------------------------------------------------------*/
+
+/* out value for the frame structure->type field
+ * unlike stats output in param2, this field is not asynchronous and tells
+ * the client app, if the frame written into the stream buffer is an ivop
+ * usually used for indexing purpose in the container */
 #define XVID_KEYFRAME (1<<1)
 
+/* The structure */
 typedef struct {
 	int version;
 
-	/* --- VOL related stuff; unless XVID_FORCEVOL is set, the encoder will not react to any changes here until the next VOL (keyframe). */
+	/* VOL related stuff
+	 * unless XVID_FORCEVOL is set, the encoder will not react to any changes
+	 * here until the next VOL (keyframe). */
+
 	int vol_flags;                     /* [in] vol flags */
 	unsigned char *quant_intra_matrix; /* [in:opt] custom intra qmatrix */
 	unsigned char *quant_inter_matrix; /* [in:opt] custom inter qmatrix */
 
-	int par;                           /* [in:opt] picture aspect ratio (refer to XVID_PAR_xxx above) */
+	int par;                           /* [in:opt] pixel aspect ratio (refer to XVID_PAR_xxx above) */
 	int par_width;                     /* [in:opt] aspect ratio width */
 	int par_height;                    /* [in:opt] aspect ratio height */
-	/* ---------------------------------------------- */
+
+	/* Other fields that can change on a frame base */
 
 	int fincr;                         /* [in:opt] framerate increment, for variable framerate only */
 	int vop_flags;                     /* [in] (general)vop-based flags */
@@ -641,39 +763,10 @@ typedef struct {
 	int out_flags;                     /* [out] bitstream output flags */
 } xvid_enc_frame_t;
 
-
-/* XVID_ENC_ENCODE param2 (optional)
-	xvid_enc_stats_t describes individual frame details
-
-	coding_type==XVID_TYPE_NOTHING if the stats are not given
-*/
-typedef struct {
-	int version;
-
-	/* encoding parameters */
-	int type;      /* [out] coding type */
-	int quant;     /* [out] frame quantizer */
-	int vol_flags; /* [out] vol flags (see above) */
-	int vop_flags; /* [out] vop flags (see above) */
-
-	/* bitrate */
-	int length;    /* [out] frame length */
-
-	int hlength;   /* [out] header length (bytes) */
-	int kblks;     /* [out] number of blocks compressed as Intra */
-	int mblks;     /* [out] number of blocks compressed as Inter */
-	int ublks;     /* [out] number of blocks marked as not_coded */
-
-	int sse_y;     /* [out] Y plane's sse */
-	int sse_u;     /* [out] U plane's sse */
-	int sse_v;     /* [out] V plane's sse */
-} xvid_enc_stats_t;
-
 #ifdef __cplusplus
 }
 #endif
 
-
 #endif
 
 /*
